@@ -47,8 +47,8 @@ directory; this can be overridden by providing the I<DESTDIR> argument.
 
 The information about where the repository is available is expected to be found
 in B<Vcs-*> fields available in the source package record. For example, the B<vim>
-package exposes such information with a field like S<B<Vcs-Git:
-git://git.debian.org/git/pkg-vim/vim.git>>, you can see it by grepping through
+package exposes such information with a field like S<B<Vcs-Hg:
+http://hg.debian.org/hg/pkg-vim/vim>>, you can see it by grepping through
 B<apt-cache showsrc vim>.
 
 If more than one source package record containing B<Vcs-*> fields is available,
@@ -223,7 +223,7 @@ B<debcheckout> and this manpage have been written by Stefano Zacchiroli
 use feature 'switch';
 use strict;
 use warnings;
-use Getopt::Long;
+use Getopt::Long qw(:config gnu_getopt);
 use Pod::Usage;
 use File::Basename;
 use File::Copy qw/copy/;
@@ -425,6 +425,14 @@ sub set_auth($$$$) {
     $user_local =~ s|(.*)(@)|$1|;
     my $user_url = $url;
 
+    # Adjust urls from new-style anonymous access to old-style and then deal
+    # with adjusting for authentication
+    $url =~ s@anonscm\.debian\.org/bzr@bzr.debian.org@;
+    $url =~ s@anonscm\.debian\.org/darcs@darcs.debian.org/darcs@;
+    $url =~ s@git://anonscm\.debian\.org@git://git.debian.org@;
+    $url =~ s@anonscm\.debian\.org/git@git.debian.org/git@;
+    $url =~ s@anonscm\.debian\.org/hg@hg.debian.org/hg@;
+    $url =~ s@svn://anonscm\.debian\.org@svn://svn.debian.org@;
     given ($repo_type) {
 	when ("bzr") {
 	    $url =~ s|^[\w+]+://(bzr\.debian\.org)/(.*)|bzr+ssh://$user$1/bzr/$2|;
@@ -819,12 +827,12 @@ sub unpack_source($$$$$) {
 	return 0;
     }
 
-    # put orig.tar.gz in place
-    my @origtgz = glob "$tmpdir/${pkg}_*.orig.tar.gz";
-    if (@origtgz) {
-	my $base = $origtgz[0];
+    # put source package in place
+    foreach my $sourcefile (glob "$tmpdir/${pkg}_*") {
+	next unless (-f $sourcefile); # skip directories
+	my $base = $sourcefile;
 	$base =~ s!.*/!!;
-	rename $origtgz[0], $base or die "rename $origtgz[0] $base: $!";
+	rename $sourcefile, $base or die "rename $sourcefile $base: $!";
     }
 
     return 1 if ($unpack_source eq 'download-only' or not $debian_only);
@@ -846,8 +854,8 @@ sub unpack_source($$$$$) {
 	} elsif ($file eq '.' or $file eq '..') {
 	    next;
 	} else {
-	    rename "$directory/$file", "$pkg/$file" or
-		die "rename $directory/$file $pkg/$file: $!";
+	    rename "$directory/$file", "$destdir/$file" or
+		die "rename $directory/$file $destdir/$file: $!";
 	}
     }
     closedir DIR;
@@ -921,6 +929,8 @@ sub guess_repo_type($$) {
 	$repo_type = $1;
     } elsif ($repo_url =~ /^https?:\/\/(svn|git|hg|bzr|darcs)\.debian\.org/) {
 	$repo_type = $1;
+    } elsif ($repo_url =~ m@^https?://anonscm.debian.org/(svn|git|hg|bzr|darcs)/@) {
+	$repo_type = $1;
     }
     return $repo_type;
 }
